#!/usr/bin/perl

use Test::More tests => 3;

use_ok('IkiWiki::Plugin::shortcuts');

my $text = <<EOF;
I need the next packages backporte to Debian stable for Ikiwiki:
* [[shortcut deb ikiwiki]]
* [[shortcut deb libcgi-formbuilder-perl]]

Also i need resolved the bug number [[shortcut dbug 19289]] in the [[shortcut
deb debootstrap]] package.

[[wiki HTTP]] is a simple protocol. 

I need write literal example \\[[shortcut technorati perl]].

Also, a [[google "ikiwiki shortcuts" "fast search in google"]] produce a few
results, and a [[cpan "ikiwiki shortcuts" "search in the CPAN archive"]]
produce no results.
EOF

my $page = <<EOF;

This is my shortcuts list:

* CPAN          http://search.cpan.org/search?mode=dist&query=
* dmoz          http://search.dmoz.org/cgi-bin/search?search=
* docbook       http://www.docbook.org/tdg/en/html/%s.html
* technorati    http://technorati.com/tag/
* foldoc        http://foldoc.doc.ic.ac.uk/foldoc/foldoc.cgi?query=
* freshmeat     http://freshmeat.net/search/?q=
* rfc           http://www.ietf.org/rfc/rfc%s.txt
* netcraft      http://uptime.netcraft.com/up/graph?site=

EOF

IkiWiki::Plugin::shortcuts::_scan_config_file(\%IkiWiki::Plugin::shortcuts::shortcuts,
                $page);

my $content = search_preprocess($text);

my $counter = 0;
$counter ++ while ($content =~ s{http}{XXXX}xms);

ok($counter == 7, 'replace shortcuts');
ok($content =~ m{technorati\s+perl}xms, 'literal example');

sub search_preprocess {
    my  $content    =   shift;

    $content =~
    s{(\\?)\[\[(\w+)\s+((?:(?:\w+=)?(?:""".*?"""|"[^"]+"|[^\s\]]+)\s*)*)\]\]}{filter_params($1, $2, $3)}seg;

    return $content;
}

sub filter_params {
    my  $escape     =   shift;
    my  $command    =   shift;
    my  $params     =   shift;

    if (length $escape) {
        return "[[$command $params]]";
    }

    my @params;
    while ($params =~ /(?:(\w+)=)?(?:"""(.*?)"""|"([^"]+)"|(\S+))(?:\s+|$)/sg) {
            my $key=$1;
            my $val;
            if (defined $2) {
                    $val=$2;
                    $val=~s/\r\n/\n/mg;
                    $val=~s/^\n+//g;
                    $val=~s/\n+$//g;
            }
            elsif (defined $3) {
                    $val=$3;
            }
            elsif (defined $4) {
                    $val=$4;
            }

            if (defined $key) {
                    push @params, $key, $val;
            }
            else {
                    push @params, $val, '';
            }
        }

    if ($command eq 'shortcut') {
        return IkiWiki::Plugin::shortcuts::preprocess(@params);
    }        
    else {
        my $handler =
            IkiWiki::Plugin::shortcuts::_build_preprocessor_code($command);
        
        return $handler->(@params);
    }      
}

